import multiprocessing
import unittest
import subprocess
import time
import signal
import sys
import os

import wexpect
from tests import PexpectTestCase

# Many of these test cases blindly assume that sequential directory
# listings of the /bin directory will yield the same results.
# This may not be true, but seems adequate for testing now.
# I should fix this at some point.

PYTHONBINQUOTE = '"{}"'.format(sys.executable)
FILTER=''.join([(len(repr(chr(x)))==3) and chr(x) or '.' for x in range(256)])
def hex_dump(src, length=16):
    result=[]
    for i in range(0, len(src), length):
       s = src[i:i+length]
       hexa = ' '.join(["%02X"%ord(x) for x in s])
       printable = s.translate(FILTER)
       result.append("%04X   %-*s   %s\n" % (i, length*3, hexa, printable))
    return ''.join(result)

def hex_diff(left, right):
        diff = ['< %s\n> %s' % (_left, _right,) for _left, _right in zip(
            hex_dump(left).splitlines(), hex_dump(right).splitlines())
            if _left != _right]
        return '\n' + '\n'.join(diff,)


class ExpectTestCase (PexpectTestCase.PexpectTestCase):

    def test_expect_basic (self):
        p = wexpect.spawn('cat', timeout=5)
        p.sendline ('Hello')
        p.sendline ('there')
        p.sendline ('Mr. Python')
        p.expect ('Hello')
        p.expect ('there')
        p.expect ('Mr. Python')
        p.sendeof ()
        p.expect (wexpect.EOF)

    def test_expect_exact_basic (self):
        p = wexpect.spawn('cat', timeout=5)
        p.sendline ('Hello')
        p.sendline ('there')
        p.sendline ('Mr. Python')
        p.expect_exact ('Hello')
        p.expect_exact ('there')
        p.expect_exact ('Mr. Python')
        p.sendeof ()
        p.expect_exact (wexpect.EOF)

    def test_expect_ignore_case(self):
        '''This test that the ignorecase flag will match patterns
        even if case is different using the regex (?i) directive.
        '''
        p = wexpect.spawn('cat', timeout=5)
        p.sendline ('HELLO')
        p.sendline ('there')
        p.expect ('(?i)hello')
        p.expect ('(?i)THERE')
        p.sendeof ()
        p.expect (wexpect.EOF)

    def test_expect_ignore_case_flag(self):
        '''This test that the ignorecase flag will match patterns
        even if case is different using the ignorecase flag.
        '''
        p = wexpect.spawn('cat', timeout=5)
        p.ignorecase = True
        p.sendline ('HELLO')
        p.sendline ('there')
        p.expect ('hello')
        p.expect ('THERE')
        p.sendeof ()
        p.expect (wexpect.EOF)

    def test_expect_order (self):
        '''This tests that patterns are matched in the same order as given in the pattern_list.

        (Or does it?  Doesn't it also pass if expect() always chooses
        (one of the) the leftmost matches in the input? -- grahn)
        ... agreed! -jquast, the buffer ptr isn't forwarded on match, see first two test cases
        '''
        p = wexpect.spawn('cat', timeout=5, echo=True)
        self._expect_order(p)

    def test_expect_order_exact (self):
        '''Like test_expect_order(), but using expect_exact().
        '''
        p = wexpect.spawn('cat', timeout=5, echo=True)
        p.expect = p.expect_exact
        self._expect_order(p)

    def _expect_order (self, p):
        p.sendline ('1234')
        p.sendline ('abcd')
        p.sendline ('wxyz')
        p.sendline ('7890')
        p.sendeof ()
        for _ in range(2):
            index = p.expect ([
                '1234',
                'abcd',
                'wxyz',
                wexpect.EOF,
                '7890' ])
            self.assertEqual(index,  0, (index, p.before, p.after))

        for _ in range(2):
            index = p.expect ([
                '54321',
                wexpect.TIMEOUT,
                '1234',
                'abcd',
                'wxyz',
                wexpect.EOF], timeout=5)
            self.assertEqual(index,  3, (index, p.before, p.after))

        for _ in range(2):
            index = p.expect ([
                '54321',
                wexpect.TIMEOUT,
                '1234',
                'abcd',
                'wxyz',
                wexpect.EOF], timeout=5)
            self.assertEqual(index,  4, (index, p.before, p.after))

        for _ in range(2):
            index = p.expect ([
                wexpect.EOF,
                'abcd',
                'wxyz',
                '7890' ])
            self.assertEqual(index,  3, (index, p.before, p.after))

        index = p.expect ([
            'abcd',
            'wxyz',
            '7890',
            wexpect.EOF])
        self.assertEqual(index,  3, (index, p.before, p.after))

        # Termination of the SpawnSocket is slow. We have to wait to prevent the failure of the next test.
        if wexpect.spawn_class_name == 'SpawnSocket':
            p.wait()

    def test_expect_index (self):
        '''This tests that mixed list of regex strings, TIMEOUT, and EOF all
        return the correct index when matched.
        '''
        p = wexpect.spawn('cat', timeout=5, echo=False)
        self._expect_index(p)

    def test_expect_index_exact (self):
        '''Like test_expect_index(), but using expect_exact().
        '''
        p = wexpect.spawn('cat', timeout=5, echo=False)
        p.expect = p.expect_exact
        self._expect_index(p)
        # Termination of the SpawnSocket is slow. We have to wait to prevent the failure of the next test.
        if wexpect.spawn_class_name == 'SpawnSocket':
            p.wait()

    def _expect_index (self, p):
        p.sendline ('1234')
        index = p.expect (['abcd','wxyz','1234',wexpect.EOF])
        self.assertEqual(index,  2, "index="+str(index))
        p.sendline ('abcd')
        index = p.expect ([wexpect.TIMEOUT,'abcd','wxyz','1234',wexpect.EOF])
        self.assertEqual(index,  1, "index="+str(index))
        p.sendline ('wxyz')
        index = p.expect (['54321',wexpect.TIMEOUT,'abcd','wxyz','1234',wexpect.EOF])
        self.assertEqual(index,  3, "index="+str(index)) # Expect 'wxyz'
        p.sendline ('$*!@?')
        index = p.expect (['54321',wexpect.TIMEOUT,'abcd','wxyz','1234',wexpect.EOF],
                          timeout=1)
        self.assertEqual(index,  1, "index="+str(index)) # Expect TIMEOUT
        p.sendeof ()
        index = p.expect (['54321',wexpect.TIMEOUT,'abcd','wxyz','1234',wexpect.EOF])
        self.assertEqual(index,  5, "index="+str(index)) # Expect EOF

    def test_expect (self):
        the_old_way = subprocess.Popen(args=['ls', '-l'],
                stdout=subprocess.PIPE).communicate()[0].rstrip()
        p = wexpect.spawn('ls -l')
        the_new_way = ''
        while 1:
            i = p.expect (['\n', wexpect.EOF])
            the_new_way = the_new_way + p.before
            if i == 1:
                break
        the_new_way = the_new_way.rstrip()
        the_new_way = the_new_way.replace('\r\n', '\n'
                ).replace('\r', '\n').replace('\n\n', '\n').rstrip()
        the_old_way = the_old_way.decode('utf-8')
        the_old_way = the_old_way.replace('\r\n', '\n'
                ).replace('\r', '\n').replace('\n\n', '\n').rstrip()
        # print(repr(the_old_way))
        # print(repr(the_new_way))
        # the_old_way = the_old_way[0:10000]
        # the_new_way = the_new_way[0:10000]
        self.assertEqual(the_old_way,  the_new_way)

    def test_expect_exact (self):
        the_old_way = subprocess.Popen(args=['ls', '-l'],
                stdout=subprocess.PIPE).communicate()[0].rstrip()
        p = wexpect.spawn('ls -l')
        the_new_way = ''
        while 1:
            i = p.expect_exact (['\n', wexpect.EOF])
            the_new_way = the_new_way + p.before
            if i == 1:
                break
        the_new_way = the_new_way.replace('\r\n', '\n'
                ).replace('\r', '\n').replace('\n\n', '\n').rstrip()
        the_old_way = the_old_way.decode('utf-8')
        the_old_way = the_old_way.replace('\r\n', '\n'
                ).replace('\r', '\n').replace('\n\n', '\n').rstrip()
        self.assertEqual(the_old_way,  the_new_way)
        # Termination of the SpawnSocket is slow. We have to wait to prevent the failure of the next test.
        if wexpect.spawn_class_name == 'SpawnSocket':
            p.wait()

        p = wexpect.spawn('echo hello.?world')
        i = p.expect_exact('.?')
        self.assertEqual(p.before, 'hello')
        self.assertEqual(p.after, '.?')

    def test_expect_eof (self):
        the_old_way = subprocess.Popen(args=['ls', '-l'],
                stdout=subprocess.PIPE).communicate()[0].rstrip()
        p = wexpect.spawn('ls -l')
        p.expect(wexpect.EOF) # This basically tells it to read everything. Same as wexpect.run() function.
        the_new_way = p.before
        the_new_way = the_new_way.replace('\r\n', '\n'
                ).replace('\r', '\n').replace('\n\n', '\n').rstrip()
        the_old_way = the_old_way.decode('utf-8')
        the_old_way = the_old_way.replace('\r\n', '\n'
                ).replace('\r', '\n').replace('\n\n', '\n').rstrip()
        self.assertEqual(the_old_way,  the_new_way)

        # Termination of the SpawnSocket is slow. We have to wait to prevent the failure of the next test.
        if wexpect.spawn_class_name == 'SpawnSocket':
            p.wait()

    def test_expect_timeout (self):
        p = wexpect.spawn('cat', timeout=5)
        p.expect(wexpect.TIMEOUT) # This tells it to wait for timeout.
        self.assertEqual(p.after, wexpect.TIMEOUT)

    def test_unexpected_eof (self):
        p = wexpect.spawn('ls -l /bin')
        try:
            p.expect('_Z_XY_XZ') # Probably never see this in ls output.
        except wexpect.EOF:
            pass
        else:
            self.fail ('Expected an EOF exception.')

        if wexpect.spawn_class_name == 'SpawnSocket':
            p.wait()

    def test_buffer_interface(self):
        p = wexpect.spawn('cat', timeout=5)
        p.sendline ('Hello')
        p.expect ('Hello')
        assert len(p.buffer)
        p.buffer = 'Testing'
        p.sendeof ()
        if wexpect.spawn_class_name == 'SpawnSocket':
            p.wait()

    def _before_after(self, p):
        p.timeout = 5

        p.expect('5')
        self.assertEqual(p.after, '5')
        self.assertTrue(p.before.startswith('[0, 1, 2'), p.before)

        p.expect('50')
        self.assertEqual(p.after, '50')
        self.assertTrue(p.before.startswith(', 6, 7, 8'), p.before[:20])
        self.assertTrue(p.before.endswith('48, 49, '), p.before[-20:])

        p.expect(wexpect.EOF)
        self.assertEqual(p.after, wexpect.EOF)
        assert p.before.startswith(', 51, 52'), p.before[:20]
        assert p.before.endswith(', 99]\r\n'), p.before[-20:]

        if wexpect.spawn_class_name == 'SpawnSocket':
            p.wait()

    def test_before_after(self):
        '''This tests expect() for some simple before/after things.
        '''
        p = wexpect.spawn('%s -Wi list100.py' % PYTHONBINQUOTE)
        self._before_after(p)

    def test_before_after_exact(self):
        '''This tests some simple before/after things, for
        expect_exact(). (Grahn broke it at one point.)
        '''
        p = wexpect.spawn('%s -Wi list100.py' % PYTHONBINQUOTE)
        # mangle the spawn so we test expect_exact() instead
        p.expect = p.expect_exact
        self._before_after(p)

    def _ordering(self, p):
        p.timeout = 20
        p.expect('>>> ')

        p.sendline('list(range(4*3))')
        self.assertEqual(p.expect(['5,', '5,']), 0)
        p.expect('>>> ')

        p.sendline('list(range(4*3))')
        self.assertEqual(p.expect(['7,', '5,']), 1)
        p.expect('>>> ')

        p.sendline('list(range(4*3))')
        self.assertEqual(p.expect(['5,', '7,']), 0)
        p.expect('>>> ')

        p.sendline('list(range(4*5))')
        self.assertEqual(p.expect(['2,', '12,']), 0)
        p.expect('>>> ')

        p.sendline('list(range(4*5))')
        self.assertEqual(p.expect(['12,', '2,']), 1)

        p.sendline('exit()')

    def test_ordering(self):
        '''This tests expect() for which pattern is returned
        when many may eventually match. I (Grahn) am a bit
        confused about what should happen, but this test passes
        with wexpect 2.1.
        '''
        p = wexpect.spawn(PYTHONBINQUOTE)
        self._ordering(p)

    def test_ordering_exact(self):
        '''This tests expect_exact() for which pattern is returned
        when many may eventually match. I (Grahn) am a bit
        confused about what should happen, but this test passes
        for the expect() method with wexpect 2.1.
        '''
        p = wexpect.spawn(PYTHONBINQUOTE)
        # mangle the spawn so we test expect_exact() instead
        p.expect = p.expect_exact
        self._ordering(p)

        # Termination of the SpawnSocket is slow. We have to wait to prevent the failure of the next test.
        if wexpect.spawn_class_name == 'SpawnSocket':
            p.wait()

    def _greed(self, expect):
        # End at the same point: the one with the earliest start should win
        self.assertEqual(expect(['3, 4', '2, 3, 4']), 1)

        # Start at the same point: first pattern passed wins
        self.assertEqual(expect(['5,', '5, 6']), 0)

        # Same pattern passed twice: first instance wins
        self.assertEqual(expect(['7, 8', '7, 8, 9', '7, 8']), 0)

    def _greed_read1(self, expect):
        # Here, one has an earlier start and a later end. When processing
        # one character at a time, the one that finishes first should win,
        # because we don't know about the other match when it wins.
        # If maxread > 1, this behaviour is currently undefined, although in
        # most cases the one that starts first will win.
        self.assertEqual(expect(['1, 2, 3', '2,']), 1)

    def test_greed(self):
        p = wexpect.spawn(PYTHONBINQUOTE + ' list100.py')
        self._greed(p.expect)

    def test_greed_exact(self):
        p = wexpect.spawn(PYTHONBINQUOTE + ' list100.py')
        self._greed(p.expect_exact)

    def test_bad_arg(self):
        p = wexpect.spawn('cat')
        with self.assertRaisesRegex(TypeError, '.*must be one of'):
            p.expect(1)
        with self.assertRaisesRegex(TypeError, '.*must be one of'):
            p.expect([1, '2'])
        with self.assertRaisesRegex(TypeError, '.*must be one of'):
            p.expect_exact(1)
        with self.assertRaisesRegex(TypeError, '.*must be one of'):
            p.expect_exact([1, '2'])

    def test_timeout_none(self):
        p = wexpect.spawn('echo abcdef', timeout=None)
        p.expect('abc')
        p.expect_exact('def')
        p.expect(wexpect.EOF)

if __name__ == '__main__':
    unittest.main()

suite = unittest.makeSuite(ExpectTestCase, 'test')



'''
This is the simplest example. It starts a windows command interpreter (aka. cmd) lists the current
directory and exits.
'''

import wexpect

# Start cmd as child process
child = wexpect.spawn('cmd.exe')

# Wait for prompt when cmd becomes ready.
child.expect('>')

# Prints the cmd's start message
print(child.before, end='')
print(child.after, end='')

# run list directory command
child.sendline('ls')

# Waiting for prompt
child.expect('>')

# Prints content of the directory
print(child.before, end='')
print(child.after, end='')

# Exit from cmd
child.sendline('exit')

# Waiting for cmd termination.
child.wait()



import wexpect
import time
import unittest
import win32process
import win32api
import os
from tests import PexpectTestCase


if "RUN_CONSOLE_READER_TEST" not in os.environ:
    skip = True

class ConsoleReaderTestCase(PexpectTestCase.PexpectTestCase):

    @unittest.skipIf(skip, "Skipping test")
    def test_console_reader(self):
        
    
        pid = win32process.GetCurrentProcessId()
        tid = win32api.GetCurrentThreadId()
        args = ['sleep', '1']
        
        with self.assertRaises(SystemExit):
            wexpect.ConsoleReader(wexpect.join_args(args), tid=tid, pid=pid, cp=1250, logdir='wexpect')

        os.system('cls')
            
if __name__ == '__main__':
    unittest.main()

suite = unittest.makeSuite(ConsoleReaderTestCase,'test')



# A simple example code for wexpect

from __future__ import print_function

import sys
import os
import re

here = os.path.dirname(os.path.abspath(__file__))
wexpectPath = os.path.dirname(here)

import wexpect

# Path of cmd executable:
cmd_exe = 'cmd'
# The prompt should be more sophisticated than just a '>'.
cmdPrompt = re.compile('[A-Z]\:.+>')

# Start the child process
p = wexpect.spawn(cmd_exe)

# Wait for prompt
p.expect(cmdPrompt, timeout = 5)

# print the texts
print(p.before, end='')
print(p.match.group(0), end='')


while True:

    # Wait and run a command.
    command = input()
    p.sendline(command)
    
    try:
        # Wait for prompt
        p.expect(cmdPrompt)
        
        # print the texts
        print(p.before, end='')
        print(p.match.group(0), end='')
    
    except wexpect.EOF:
        # The program has exited
        print('The program has exied... BY!')
        break



import wexpect
import unittest
import sys
import os
import time
from tests import PexpectTestCase

@unittest.skipIf(wexpect.spawn_class_name == 'legacy_wexpect', "legacy unsupported")
class TestCaseParametricPrinter(PexpectTestCase.PexpectTestCase):
    def test_all_line_length (self):

        here = os.path.dirname(os.path.abspath(__file__))
        sys.path.insert(0, here)

        # With quotes (C:\Program Files\Python37\python.exe needs quotes)
        python_executable = '"' + sys.executable + '" '
        child_script = here + '\\parametric_printer.py'

        self.prompt = '> '

        # Start the child process
        self.p = wexpect.spawn(python_executable + ' '  + child_script, coverage_console_reader=True)
        # Wait for prompt
        self.p.expect(self.prompt)

        self._test(['a'], range(1,200), [1], [0])

        self.p.terminate()

    def test_long_console(self):

        here = os.path.dirname(os.path.abspath(__file__))
        sys.path.insert(0, here)

        # With quotes (C:\Program Files\Python37\python.exe needs quotes)
        python_executable = '"' + sys.executable + '" '
        child_script = here + '\\parametric_printer.py'

        self.prompt = '> '

        # Start the child process
        self.p = wexpect.spawn(python_executable + ' '  + child_script, coverage_console_reader=True)
        # Wait for prompt
        self.p.expect(self.prompt)

        self._test(['a', 'b', 'c', 'd', 'e', 'f'], [8, 16, 32, 64], [64, 128, 256], [-1, 0])

        self.p.terminate()

    def _test(self, character_list, character_count_list, line_count_list, speed_ms_list):

        # print(f'character_list: {character_list}  character_count_list: {character_count_list}  line_count_list: {line_count_list}  speed_ms_list: {speed_ms_list}')
        for character in character_list:
            for character_count in character_count_list:
                for line_count in line_count_list:
                    for speed_ms in speed_ms_list:
                        command = f'{character},{character_count},{line_count},{speed_ms}'
                        self.p.sendline(command)
                        self.p.expect(self.prompt)
                        expected = [character*character_count] * line_count
                        try:
                            self.assertEqual(self.p.before.splitlines()[1:-1], expected)
                        except:
                            raise

if __name__ == '__main__':
    unittest.main()

suite = unittest.makeSuite(TestCaseParametricPrinter,'test')



import argparse
import sys
import logging
import traceback

import wexpect.console_reader as console_reader
import wexpect.wexpect_util as wexpect_util


logger = logging.getLogger('wexpect')
logger.info('Hello')

def main():
    try:
        parser = argparse.ArgumentParser(description='Wexpect: executable automation for Windows.')

        parser.add_argument('--console_reader_class', type=str,
                            help='Class name of the console reader class.')

        parser.add_argument('command', type=str, nargs='+', help='Command to be run with its arguments')
        parser.add_argument('--host_pid', type=int, help='Host process process ID')
        parser.add_argument('--codepage', type=str, help='Codepage')
        parser.add_argument('--window_size_x', type=int, help='Width of the console window', default=80)
        parser.add_argument('--window_size_y', type=int, help='Height of the console window', default=25)
        parser.add_argument('--buffer_size_x', type=int, help='Width of the console buffer', default=80)
        parser.add_argument('--buffer_size_y', type=int, help='Height of the console buffer',
                            default=16000)
        parser.add_argument('--local_echo', type=str, help='Echo sent characters', default=True)
        parser.add_argument('--interact', type=str, help='Show console window', default=False)
        parser.add_argument('--port', type=int, help=
                            "If the console reader class is SpawnSocket, this option specifies the "
                            "socket's port.", default=False)
        parser.add_argument('--pipe_file_name', type=str, help=
                            "If the console reader class is SpawnPipe, this option specifies the "
                            "pipe's name.", default=None)

        try:
            args = parser.parse_args()
        except SystemExit: # pragma: no cover
            logger.error('Unexpected exception.')
            logger.info(traceback.format_exc())
            raise

        logger.info(f'Starter arguments: {args}')

        if args.console_reader_class == 'ConsoleReaderSocket':
            conole_reader_class = console_reader.ConsoleReaderSocket
        elif args.console_reader_class == 'ConsoleReaderPipe':
            conole_reader_class = console_reader.ConsoleReaderPipe

        command = wexpect_util.join_args(args.command)

        cons = conole_reader_class(
            path=command, host_pid=args.host_pid, codepage=args.codepage, port=args.port, pipe_file_name=args.pipe_file_name,
            window_size_x=args.window_size_x, window_size_y=args.window_size_y,
            buffer_size_x=args.buffer_size_x, buffer_size_y=args.buffer_size_y,
            local_echo=wexpect_util.str2bool(args.local_echo), interact=wexpect_util.str2bool(args.interact))

        logger.info(f'Exiting with status: {cons.child_exitstatus}')
        sys.exit(cons.child_exitstatus)

    except Exception as e: # pragma: no cover
        logger.error('Unexpected exception.')
        logger.info(traceback.format_exc())
        raise

if __name__ == "__main__":
    main()



import wexpect
import unittest
import time
from tests import PexpectTestCase

class TestCaseConstructor(PexpectTestCase.PexpectTestCase):
    def test_constructor (self):
        '''This tests that the constructor will work and give
        the same results for different styles of invoking __init__().
        This assumes that the root directory / is static during the test.
        '''
        p1 = wexpect.spawn('uname -m -n -p -r -s -v', timeout=5)
        p1.expect(wexpect.EOF)
        time.sleep(p1.delayafterterminate)
        p2 = wexpect.spawn('uname', ['-m', '-n', '-p', '-r', '-s', '-v'], timeout=5)
        p2.expect(wexpect.EOF)
        time.sleep(p1.delayafterterminate)
        self.assertEqual(p1.before, p2.before)
        self.assertEqual(str(p1).splitlines()[1:9], str(p2).splitlines()[1:9])
        
        run_result = wexpect.run('uname -m -n -p -r -s -v')
        self.assertEqual(run_result, p2.before)

    def test_named_parameters (self):
        '''This tests that named parameters work.
        '''
        p = wexpect.spawn('ls',timeout=10)
        p.wait()
        p = wexpect.spawn(timeout=10, command='ls')
        p.wait()
        p = wexpect.spawn(args=[], command='ls')
        p.wait()

if __name__ == '__main__':
    unittest.main()

suite = unittest.makeSuite(TestCaseConstructor,'test')



'''
This is is a very basic stdio handler script. This is used by python.py example.
'''

import time


print('Welcome!')

while True:
    print('>', end='')
    cmd = input()

    # Wait the given time
    if cmd == 'ls':
        print('apple\tbanana\tmelone\tcherry')
        print('pepper tomato\tcorn cabbage')
        print('apple banana\tmelone\tcherry2')
        print('pepper tomato\tcorn cabbage2')
        continue
    if cmd == 'exit':
        break
    else:
        print('unknown command')


"""Wexpect is a Windows variant of pexpect https://pexpect.readthedocs.io.

Wexpect is a Python module for spawning child applications and controlling
them automatically.

console_reader Implements a virtual terminal, and starts the child program.
The main wexpect.spawn class connect to this class to reach the child's terminal.
"""

import time
import logging
import os
import traceback
import psutil
from io import StringIO

import ctypes
from ctypes import windll
import win32console
import win32process
import win32con
import win32file
import win32gui
import win32pipe
import socket

from .wexpect_util import init_logger
from .wexpect_util import EOF_CHAR
from .wexpect_util import SIGNAL_CHARS
from .wexpect_util import TIMEOUT

#
# System-wide constants
#
screenbufferfillchar = '\4'
maxconsoleY = 8000
default_port = 4321

#
# Create logger: We write logs only to file. Printing out logs are dangerous, because of the deep
# console manipulation.
#
logger = logging.getLogger('wexpect')


class ConsoleReaderBase:
    """Consol class (aka. client-side python class) for the child.

    This class initialize the console starts the child in it and reads the console periodically.
    """

    def __init__(self, path, host_pid, codepage=None, window_size_x=80, window_size_y=25,
                 buffer_size_x=80, buffer_size_y=16000, local_echo=True, interact=False, **kwargs):
        """Initialize the console starts the child in it and reads the console periodically.

        Args:
            path (str): Child's executable with arguments.
            parent_pid (int): Parent (aka. host) process process-ID
            codepage (:obj:, optional): Output console code page.
        """
        self.lastRead = 0
        self.__bufferY = 0
        self.lastReadData = ""
        self.totalRead = 0
        self.__buffer = StringIO()
        self.__currentReadCo = win32console.PyCOORDType(0, 0)
        self.pipe = None
        self.connection = None
        self.consin = None
        self.consout = None
        self.local_echo = local_echo
        self.console_pid = os.getpid()
        self.host_pid = host_pid
        self.host_process = psutil.Process(host_pid)
        self.child_process = None
        self.child_pid = None
        self.enable_signal_chars = True
        self.timeout = 30
        self.child_exitstatus = None

        logger.info(f'ConsoleReader started. location {os.path.abspath(__file__)}')

        if codepage is None:
            codepage = windll.kernel32.GetACP()

        try:
            logger.info("Setting console output code page to %s" % codepage)
            win32console.SetConsoleOutputCP(codepage)
            logger.info(
                "Console output code page: %s" % ctypes.windll.kernel32.GetConsoleOutputCP())
        except Exception as e: # pragma: no cover
            # I hope this code is unreachable...
            logger.error(e)

        try:
            self.create_connection(**kwargs)
            logger.info('Spawning %s' % path)
            try:
                self.initConsole()
                self.child_process = psutil.Popen(path)

                logger.info(f'Child pid: {self.child_pid}  Console pid: {self.console_pid}')

            except Exception: # pragma: no cover
                # I hope this code is unreachable...
                logger.error(traceback.format_exc())
                return

            if interact:
                self.interact()
                self.interact()

            self.read_loop()
        except Exception: # pragma: no cover
            # I hope this code is unreachable...
            logger.error(traceback.format_exc())
        finally:
            try:
                self.terminate_child()
                time.sleep(.01)
                self.send_to_host(self.readConsoleToCursor())
                self.sendeof()
                time.sleep(.1)
                self.close_connection()
                logger.info('Console finished.')
            except Exception: # pragma: no cover
                # I hope this code is unreachable...
                logger.error(traceback.format_exc())

    def read_loop(self):

        while True:
            if not self.isalive(self.host_process):
                logger.info('Host process has been died.')
                return

            try:
                self.child_exitstatus = self.child_process.wait(0)
                logger.info(f'Child finished with code: {self.child_exitstatus}')
                return
            except psutil.TimeoutExpired:
                pass

            consinfo = self.consout.GetConsoleScreenBufferInfo()
            cursorPos = consinfo['CursorPosition']

            if cursorPos.Y > maxconsoleY:
                '''If the console output becomes long, we suspend the child, read all output then
                clear the console before we resume the child.
                '''
                logger.info('cursorPos %s' % cursorPos)
                self.suspend_child()
                time.sleep(.2)
                self.send_to_host(self.readConsoleToCursor())
                self.refresh_console()
                self.resume_child()
            else:
                self.send_to_host(self.readConsoleToCursor())

            s = self.get_from_host()
            if s:
                logger.debug(f'get_from_host: {s}')
            else:
                logger.spam(f'get_from_host: {s}')
            if self.enable_signal_chars:
                for sig, char in SIGNAL_CHARS.items():
                    if char in s:
                        self.child_process.send_signal(sig)
            s = s.decode()
            self.write(s)


            time.sleep(.02)

    def suspend_child(self):
        """Pauses the main thread of the child process."""
        handle = windll.kernel32.OpenThread(win32con.THREAD_SUSPEND_RESUME, 0, self.child_tid)
        win32process.SuspendThread(handle)

    def resume_child(self):
        """Un-pauses the main thread of the child process."""
        handle = windll.kernel32.OpenThread(win32con.THREAD_SUSPEND_RESUME, 0, self.child_tid)
        win32process.ResumeThread(handle)

    def refresh_console(self):
        """Clears the console after pausing the child and
        reading all the data currently on the console."""

        orig = win32console.PyCOORDType(0, 0)
        self.consout.SetConsoleCursorPosition(orig)
        self.__currentReadCo.X = 0
        self.__currentReadCo.Y = 0
        writelen = self.__consSize.X * self.__consSize.Y
        # Use NUL as fill char because it displays as whitespace
        # (if we interact() with the child)
        self.consout.FillConsoleOutputCharacter(screenbufferfillchar, writelen, orig)

        self.__bufferY = 0
        self.__buffer.truncate(0)

    def terminate_child(self):
        try:
            if self.child_process:
                self.child_process.kill()
        except psutil.NoSuchProcess:
            logger.info('The process has already died.')
        return

    def isalive(self, process):
        """True if the child is still alive, false otherwise"""
        try:
            self.exitstatus = process.wait(timeout=0)
            return False
        except psutil.TimeoutExpired:
            return True

    def write(self, s):
        """Writes input into the child consoles input buffer."""

        if len(s) == 0:
            return 0
        if s[-1] == '\n' or s[-1] == "\r":
            s = s[:-1]
        records = [self.createKeyEvent(c) for c in str(s)]
        if not self.consout:
            return ""

        # Store the current cursor position to hide characters in local echo disabled mode
        # (workaround).
        consinfo = self.consout.GetConsoleScreenBufferInfo()
        startCo = consinfo['CursorPosition']

        # Send the string to console input
        wrote = self.consin.WriteConsoleInput(records)

        # Wait until all input has been recorded by the console.
        ts = time.time()
        while self.consin.PeekConsoleInput(8) != ():
            if time.time() > ts + len(s) * .1 + .5:
                break
            time.sleep(.05)

        # Hide characters in local echo disabled mode (workaround).
        if not self.local_echo:
            self.consout.FillConsoleOutputCharacter(screenbufferfillchar, len(s), startCo)

        return wrote

    def createKeyEvent(self, char):
        """Creates a single key record corrosponding to
            the ascii character char."""

        evt = win32console.PyINPUT_RECORDType(win32console.KEY_EVENT)
        evt.KeyDown = True
        evt.Char = char
        evt.RepeatCount = 1
        return evt

    def initConsole(self, consout=None, window_size_x=80, window_size_y=25, buffer_size_x=80,
                    buffer_size_y=16000):
        if not consout:
            consout = self.getConsoleOut()

        self.consin = win32console.GetStdHandle(win32console.STD_INPUT_HANDLE)

        rect = win32console.PySMALL_RECTType(0, 0, window_size_x - 1, window_size_y - 1)
        consout.SetConsoleWindowInfo(True, rect)
        size = win32console.PyCOORDType(buffer_size_x, buffer_size_y)
        consout.SetConsoleScreenBufferSize(size)
        pos = win32console.PyCOORDType(0, 0)
        # Use NUL as fill char because it displays as whitespace
        # (if we interact() with the child)
        consout.FillConsoleOutputCharacter(screenbufferfillchar, size.X * size.Y, pos)

        consinfo = consout.GetConsoleScreenBufferInfo()
        self.__consSize = consinfo['Size']
        logger.info('self.__consSize: ' + str(self.__consSize))
        self.startCursorPos = consinfo['CursorPosition']

    def parseData(self, s):
        """Ensures that special characters are interpretted as
        newlines or blanks, depending on if there written over
        characters or screen-buffer-fill characters."""

        strlist = []
        for i, c in enumerate(s):
            if c == screenbufferfillchar:
                if (self.totalRead - self.lastRead + i + 1) % self.__consSize.X == 0:
                    strlist.append('\r\n')
            else:
                strlist.append(c)

        s = ''.join(strlist)
        return s

    def getConsoleOut(self):
        consfile = win32file.CreateFile(
            'CONOUT$',
            win32con.GENERIC_READ | win32con.GENERIC_WRITE,
            win32con.FILE_SHARE_READ | win32con.FILE_SHARE_WRITE,
            None,
            win32con.OPEN_EXISTING,
            0,
            0)

        self.consout = win32console.PyConsoleScreenBufferType(consfile)
        return self.consout

    def getCoord(self, offset):
        """Converts an offset to a point represented as a tuple."""

        x = offset % self.__consSize.X
        y = offset // self.__consSize.X
        return win32console.PyCOORDType(x, y)

    def getOffset(self, coord):
        """Converts a tuple-point to an offset."""

        return coord.X + coord.Y * self.__consSize.X

    def readConsole(self, startCo, endCo):
        """Reads the console area from startCo to endCo and returns it
        as a string."""

        if startCo is None:
            startCo = self.startCursorPos
            startCo.Y = startCo.Y

        if endCo is None:
            consinfo = self.consout.GetConsoleScreenBufferInfo()
            endCo = consinfo['CursorPosition']
            endCo = self.getCoord(0 + self.getOffset(endCo))

        buff = []
        self.lastRead = 0

        while True:
            startOff = self.getOffset(startCo)
            endOff = self.getOffset(endCo)
            readlen = endOff - startOff

            if readlen <= 0:
                break

            if readlen > 4000:
                readlen = 4000
            endPoint = self.getCoord(startOff + readlen)

            s = self.consout.ReadConsoleOutputCharacter(readlen, startCo)
            self.lastRead += len(s)
            self.totalRead += len(s)
            buff.append(s)

            startCo = endPoint

        return ''.join(buff)

    def readConsoleToCursor(self):
        """Reads from the current read position to the current cursor
        position and inserts the string into self.__buffer."""
        if not self.consout:
            return ""
            
        consinfo = self.consout.GetConsoleScreenBufferInfo()
        cursorPos = consinfo['CursorPosition']

        # 添加: 始终读取当前行，即使光标没有移动
        current_line_start = win32console.PyCOORDType(0, cursorPos.Y)
        try:
            current_line_content = self.consout.ReadConsoleOutputCharacter(self.__consSize.X, current_line_start)
            
            # 检查当前行内容是否发生变化
            if not hasattr(self, '_last_line_content'): 
                self._last_line_content = ""
            
            content_changed = current_line_content != self._last_line_content  
            self._last_line_content = current_line_content
        except Exception as e:
            logger.debug(f"Error reading current line: {e}")
            content_changed = False
        
        logger.spam('cursor: %r, current: %r' % (cursorPos, self.__currentReadCo))
        
        isSameX = cursorPos.X == self.__currentReadCo.X
        isSameY = cursorPos.Y == self.__currentReadCo.Y
        isSamePos = isSameX and isSameY
        
        logger.spam('isSameY: %r' % isSameY)
        logger.spam('isSamePos: %r' % isSamePos)
        logger.spam('content_changed: %r' % content_changed)
        
        # 如果内容已变化或是其他需要刷新的情况，重置读取位置
        if content_changed or isSameY or not self.lastReadData.endswith('\r\n'):
            # Read the current slice again
            self.totalRead -= self.lastRead
            self.__currentReadCo.X = 0
            self.__currentReadCo.Y = self.__bufferY
        
        logger.spam('cursor: %r, current: %r' % (cursorPos, self.__currentReadCo))
        
        raw = self.readConsole(self.__currentReadCo, cursorPos)
        rawlist = []
        while raw:
            rawlist.append(raw[:self.__consSize.X])
            raw = raw[self.__consSize.X:]
        raw = ''.join(rawlist)
        s = self.parseData(raw)
        
        for i, line in enumerate(reversed(rawlist)):
            if line.endswith(screenbufferfillchar):
                # Record the Y offset where the most recent line break was detected
                self.__bufferY += len(rawlist) - i
                break
        
        logger.spam('lastReadData: %r' % self.lastReadData)
        
        if s:
            logger.debug('Read: %r' % s)
        else:
            logger.spam('Read: %r' % s)
        
        # 只有位置相同，内容相同且当前行未变化时才返回空
        if isSamePos and self.lastReadData == s and not content_changed:
            logger.spam('isSamePos and self.lastReadData == s and not content_changed')
            s = ''
        
        if s:
            lastReadData = self.lastReadData
            pos = self.getOffset(self.__currentReadCo)
            self.lastReadData = s
            
            if isSameY or not lastReadData.endswith('\r\n'):
                # Detect changed lines
                self.__buffer.seek(pos)
                buf = self.__buffer.read()
                if raw.startswith(buf):
                    # Line has grown
                    rawslice = raw[len(buf):]
                    # Update last read bytes so line breaks can be detected in parseData
                    lastRead = self.lastRead
                    self.lastRead = len(rawslice)
                    s = self.parseData(rawslice)
                    self.lastRead = lastRead
                else:
                    # Cursor has been repositioned
                    s = '\r' + s
                    
            self.__buffer.seek(pos)
            self.__buffer.truncate()
            self.__buffer.write(raw)
        
        self.__currentReadCo.X = cursorPos.X
        self.__currentReadCo.Y = cursorPos.Y
        
        # 确保在内容变化时始终返回最新内容
        if content_changed and not s:
            s = self.parseData(current_line_content)
        
        return s

    def interact(self):
        """Displays the child console for interaction."""

        logger.debug('Start interact window')
        win32gui.ShowWindow(win32console.GetConsoleWindow(), win32con.SW_SHOW)

    def sendeof(self):
        """This sends an EOF to the host. This sends a character which inform the host that child
        has been finished, and all of it's output has been send to host.
        """

        self.send_to_host(EOF_CHAR)


class ConsoleReaderSocket(ConsoleReaderBase):

    def create_connection(self, **kwargs):
        try:
            self.port = kwargs['port']
            # Create a TCP/IP socket
            self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            server_address = ('localhost', self.port)
            self.sock.bind(server_address)
            logger.info(f'Socket started at port: {self.port}')

            # Listen for incoming connections
            self.sock.settimeout(5)
            self.sock.listen(1)
            self.connection, client_address = self.sock.accept()
            self.connection.settimeout(.01)
            logger.info(f'Client connected: {client_address}')
        except Exception as e:  # pragma: no cover
            # I hope this code is unreachable.
            logger.error(f"Port: {self.port} {e}")
            raise

    def close_connection(self):
        if self.connection:
            self.connection.shutdown(socket.SHUT_RDWR)
            self.connection.close()
            self.connection = None

    def send_to_host(self, msg):
        # convert to bytes
        if isinstance(msg, str):
            msg = str.encode(msg)
        if msg:
            logger.debug(f'Sending msg: {msg}')
        else:
            logger.spam(f'Sending msg: {msg}')
        self.connection.sendall(msg)

    def get_from_host(self):
        try:
            msg = self.connection.recv(4096)
        except socket.timeout as e:
            err = e.args[0]
            # this next if/else is a bit redundant, but illustrates how the
            # timeout exception is setup
            if err == 'timed out':
                logger.debug('recv timed out, retry later')
                return b''
            else:
                raise
        else:
            if len(msg) == 0:
                raise Exception('orderly shutdown on server end')
            else:
                # got a message do something :)
                return msg


class ConsoleReaderPipe(ConsoleReaderBase):
    def create_connection(self, timeout=-1, **kwargs):
        if timeout == -1:
            timeout = self.timeout
        if timeout is None:
            end_time = float('inf')
        else:
            end_time = time.time() + timeout

        try:
            self.pipe_name = kwargs['pipe_file_name']
        except KeyError:
            self.pipe_name = 'wexpect_{}'.format(self.console_pid)

        pipe_full_path = r'\\.\pipe\{}'.format(self.pipe_name)
        logger.info('Start pipe server: %s', pipe_full_path)
        self.pipe = win32pipe.CreateNamedPipe(
            pipe_full_path,
            win32pipe.PIPE_ACCESS_DUPLEX,
            win32pipe.PIPE_TYPE_MESSAGE | win32pipe.PIPE_READMODE_MESSAGE | win32pipe.PIPE_NOWAIT,
            1, 65536, 65536, 10000, None)
        logger.info("waiting for client")
        while True:
            if end_time < time.time():
                raise TIMEOUT('Connect to child has been timed out.')
            try:
                win32pipe.ConnectNamedPipe(self.pipe, None)
                break
            except Exception as e:
                logger.debug(e)
                time.sleep(0.2)
        logger.info('got client')

    def close_connection(self):
        if self.pipe:
            win32file.CloseHandle(self.pipe)

    def send_to_host(self, msg):
        # convert to bytes
        if isinstance(msg, str):
            msg = str.encode(msg)
        if msg:
            logger.debug(f'Sending msg: {msg}')
        else:
            logger.spam(f'Sending msg: {msg}')
        win32file.WriteFile(self.pipe, msg)

    def get_from_host(self):
        data, avail, bytes_left = win32pipe.PeekNamedPipe(self.pipe, 4096)
        logger.spam(f'data: {data}  avail:{avail}  bytes_left{bytes_left}')
        if avail > 0:
            resp = win32file.ReadFile(self.pipe, 4096)
            ret = resp[1]
            return ret
        else:
            return b''



import wexpect
import time
import sys
import os
import unittest
from tests import PexpectTestCase

here = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, here)

print(wexpect.__version__)

# With quotes (C:\Program Files\Python37\python.exe needs quotes)
python_executable = '"' + sys.executable + '" '
child_script = here + '\\lines_printer.py'

class ReadLineTestCase(PexpectTestCase.PexpectTestCase):
    def testReadline(self):
        fooPath = python_executable + ' '  + child_script
        prompt = ': '
        num = 5

        # Start the child process
        p = wexpect.spawn(fooPath)
        # Wait for prompt
        p.expect(prompt)
        p.sendline(str(num))
        p.expect('Bye!\r\n')
        expected_lines = p.before.splitlines(True) # Keep the line end
        expected_lines += [p.match.group()]

        # Termination of the SpawnSocket is slow. We have to wait to prevent the failure of the next test.
        if wexpect.spawn_class_name == 'SpawnSocket':
            p.wait()

        # Start the child process
        p = wexpect.spawn(fooPath)
        # Wait for prompt
        p.expect(prompt)

        p.sendline(str(num))
        for i in range(num +2): # +1 the line of sendline +1: Bye
            line = p.readline()
            self.assertEqual(expected_lines[i], line)

        # Termination of the SpawnSocket is slow. We have to wait to prevent the failure of the next test.
        if wexpect.spawn_class_name == 'SpawnSocket':
            p.wait()

        # Start the child process
        p = wexpect.spawn(fooPath)
        # Wait for prompt
        p.expect(prompt)

        p.sendline(str(num))
        readlines_lines = p.readlines()
        self.assertEqual(expected_lines, readlines_lines)

        # Termination of the SpawnSocket is slow. We have to wait to prevent the failure of the next test.
        if wexpect.spawn_class_name == 'SpawnSocket':
            p.wait()


if __name__ == '__main__':
    unittest.main()

suite = unittest.makeSuite(ReadLineTestCase,'test')



import wexpect
import unittest
from tests import PexpectTestCase
import gc
import platform
import time

class TestCaseDestructor(PexpectTestCase.PexpectTestCase):
    def test_destructor (self):
        if platform.python_implementation() != 'CPython':
            # Details of garbage collection are different on other implementations
            return 'SKIP'
        gc.collect()
        time.sleep(2)
        p1 = wexpect.spawn('ls', port=4321)
        p2 = wexpect.spawn('ls', port=4322)
        p3 = wexpect.spawn('ls', port=4323)
        p4 = wexpect.spawn('ls', port=4324)
        p1.expect(wexpect.EOF)
        p2.expect(wexpect.EOF)
        p3.expect(wexpect.EOF)
        p4.expect(wexpect.EOF)
        p1.kill()
        p2.kill()
        p3.kill()
        p4.kill()
        p1 = None
        p2 = None
        p3 = None
        p4 = None
        gc.collect()
        time.sleep(2) # Some platforms are slow at gc... Solaris!
        

        p1 = wexpect.spawn('ls', port=4321)
        p2 = wexpect.spawn('ls', port=4322)
        p3 = wexpect.spawn('ls', port=4323)
        p4 = wexpect.spawn('ls', port=4324)
        p1.kill()
        p2.kill()
        p3.kill()
        p4.kill()
        del (p1)
        del (p2)
        del (p3)
        del (p4)
        gc.collect()
        time.sleep(2)

        p1 = wexpect.spawn('ls', port=4321)
        p2 = wexpect.spawn('ls', port=4322)
        p3 = wexpect.spawn('ls', port=4323)
        p4 = wexpect.spawn('ls', port=4324)

    @unittest.skipIf(wexpect.spawn_class_name == 'legacy_wexpect', "legacy unsupported")
    def test_failed_termination(self):
        "Test failed termination."
        child = wexpect.spawn('cat')
        delayafterterminate_orig = child.delayafterterminate
        child.delayafterterminate =.1

        msg = 'Child has not been terminated even after it was killed.'
        with self.assertRaisesRegex(wexpect.ExceptionPexpect, msg):
            child.terminate()

        child.isalive(timeout = delayafterterminate_orig)

if __name__ == '__main__':
    unittest.main()

suite = unittest.makeSuite(TestCaseDestructor,'test')



import wexpect
import unittest
from tests import PexpectTestCase

class TestCaseSplitCommandLine(PexpectTestCase.PexpectTestCase):
    def test_split_sizes(self):
        self.assertEqual(len(wexpect.split_command_line(r'')), 0)
        self.assertEqual(len(wexpect.split_command_line(r'one')), 1)
        self.assertEqual(len(wexpect.split_command_line(r'one two')), 2)
        self.assertEqual(len(wexpect.split_command_line(r'one  two')), 2)
        self.assertEqual(len(wexpect.split_command_line(r'one   two')), 2)
        self.assertEqual(len(wexpect.split_command_line(r'one^ one')), 1)
        self.assertEqual(len(wexpect.split_command_line('\'one one\'')), 1)
        self.assertEqual(len(wexpect.split_command_line(r'one\"one')), 1)
        self.assertEqual(len(wexpect.split_command_line(r"This^' is a^'^ test")), 3)
        
    def test_join_args(self):
        cmd = 'foo bar "b a z"'
        cmd2 = wexpect.join_args(wexpect.split_command_line(cmd))
        self.assertEqual(cmd2,  cmd)
        
        cmd = ['foo', 'bar', 'b a z']
        cmd2 = wexpect.split_command_line(wexpect.join_args(cmd))
        self.assertEqual(cmd2,  cmd)

if __name__ == '__main__':
    unittest.main()

suite = unittest.makeSuite(TestCaseSplitCommandLine,'test')




import wexpect
import time
import sys
import os

here = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, here)

import i11_greek_printer

print(wexpect.__version__)

# With quotes (C:\Program Files\Python37\python.exe needs quotes)
python_executable = '"' + sys.executable + '" '
child_script = here + '\\i11_greek_printer.py'


def main():
    unicodePrinter = python_executable + ' '  + child_script
    prompt = 'give the name of a greek letter> '

    # Start the child process
    print('starting child: ' + unicodePrinter)
    p = wexpect.spawn(unicodePrinter)
    print('waiting for prompt')

    # Wait for prompt
    p.expect(prompt)
    print('Child prompt arrived, lets start commands!')

    # Send commands
    for letterName in i11_greek_printer.greekLetters.keys():
        p.sendline(letterName)
        p.expect(prompt)
        print(p.before.splitlines()[1])
        if i11_greek_printer.greekLetters[letterName] != p.before.splitlines()[1]:
            p.interact()
            time.sleep(5)
            raise Exception()
                 
        
main()
        



import wexpect
import unittest
import signal
import sys
import time
from tests import PexpectTestCase

PYBIN = '"{}"'.format(sys.executable)

class IsAliveTestCase(PexpectTestCase.PexpectTestCase):
    """Various tests for the running status of processes."""

    def test_expect_wait(self):
        """Ensure consistency in wait() and isalive()."""
        p = wexpect.spawn('sleep 1')
        assert p.isalive()
        self.assertEqual(p.wait(), 0)
        assert not p.isalive()
        # In previous versions of ptyprocess/wexpect, calling wait() a second
        # time would raise an exception, but not since v4.0
        self.assertEqual(p.wait(), 0)

    def test_signal_wait(self):
        '''Test calling wait with a process terminated by a signal.'''
        if not hasattr(signal, 'SIGALRM'):
            return 'SKIP'
        p = wexpect.spawn(PYBIN, ['alarm_die.py'])
        p.wait()
        assert p.exitstatus is None
        self.assertEqual(p.signalstatus, signal.SIGALRM)

    def test_expect_isalive_dead_after_normal_termination (self):
        p = wexpect.spawn('ls', timeout=15)
        p.expect(wexpect.EOF)
        assert not p.isalive()

    def test_expect_isalive_dead_after_SIGHUP(self):
        p = wexpect.spawn('cat', timeout=5)
        assert p.isalive()
        self.assertTrue(p.terminate())
        p.expect(wexpect.EOF)
        assert not p.isalive()

    def test_expect_isalive_dead_after_SIGINT(self):
        p = wexpect.spawn('cat', timeout=5)
        assert p.isalive()
        force = False
        if sys.platform.lower().startswith('sunos'):
            # On Solaris (SmartOs), and only when executed from cron(1), SIGKILL
            # is required to end the sub-process. This is done using force=True
            force = True
        self.assertEqual(p.terminate(force), True)
        p.expect(wexpect.EOF)
        assert not p.isalive()

    def test_expect_isalive_dead_after_SIGKILL(self):
        p = wexpect.spawn('cat', timeout=5)
        assert p.isalive()
        p.kill()
        p.expect(wexpect.EOF)
        assert not p.isalive()

    def test_forced_terminate(self):
    
        p = wexpect.spawn(PYBIN + ' needs_kill.py')
        p.expect('READY')
        self.assertEqual(p.terminate(force=True), True)
        p.expect(wexpect.EOF)
        assert not p.isalive()

### Some platforms allow this. Some reset status after call to waitpid.
### probably not necessary, isalive() returns early when terminate is False.
    def test_expect_isalive_consistent_multiple_calls (self):
        '''This tests that multiple calls to isalive() return same value.
        '''
        p = wexpect.spawn('cat')
        assert p.isalive()
        assert p.isalive()
        p.sendeof()
        p.expect(wexpect.EOF)
        assert not p.isalive()
        assert not p.isalive()

if __name__ == '__main__':
    unittest.main()

suite = unittest.makeSuite(IsAliveTestCase, 'test')



import wexpect
import time
import sys
import os

here = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, here)

from long_printer import puskas_wiki

print(wexpect.__version__)


# With quotes (C:\Program Files\Python37\python.exe needs quotes)
python_executable = '"' + sys.executable + '" '
child_script = here + '\\long_printer.py'


def main():
    longPrinter = python_executable + ' '  + child_script
    prompt = 'puskas> '
    
    # Start the child process
    p = wexpect.spawn(longPrinter)
    # Wait for prompt
    p.expect(prompt)
    
    try:
        for i in range(10):
            print('.', end='')
            p.sendline('0')
            p.expect(prompt)
            if p.before.splitlines()[1] != puskas_wiki[0]:
                print(p.before.splitlines()[1])
                raise Exception()
                
            p.sendline('all')
            p.expect(prompt)
            for a,b in zip(p.before.splitlines()[1:], puskas_wiki):
                if a!=b:
                    print(a)
                    print(b)
                    raise Exception()
                    
            for j, paragraph in enumerate(puskas_wiki):
                p.sendline(str(j))
                p.expect(prompt)
                if p.before.splitlines()[1] != paragraph:
                    print(p.before.splitlines()[1])
                    print(i)
                    print(j)
                    print(paragraph)
                    raise Exception()
    except:
        p.interact()
        time.sleep(5)
    else:
        print('')
        print('[PASS]')
        
            
main()
        



'''
This is is a very basic stdio handler script. This is used by python.py example.
'''

import time
import sys

# Read an integer from the user:

while True:
    print('Format:character,character_count,line_count,speed_ms> ', end='')
    command = input()
    (character,character_count,line_count,speed_ms) = command.split(',')
    character_count = int(character_count)
    speed_ms = int(speed_ms)
    line_count = int(line_count)

    if line_count<1:
        sys.exit(0)
    for _ in range(line_count):
        if speed_ms<0:
            print(character*character_count)
            sys.stdout.flush()
        else:
            for i in range(character_count):
                if i == character_count-1:
                    print(character)
                    sys.stdout.flush()
                else:
                    print(character, end='')
                    sys.stdout.flush()
                time.sleep(speed_ms/1000)



import wexpect
import unittest
import subprocess
import tempfile
import sys
import os
import re
from tests import PexpectTestCase

unicode_type = str


def timeout_callback(values):
    if values["event_count"] > 3:
        return 1
    return 0


def function_events_callback(values):
    try:
        previous_echoed = values["child_result_list"][-1]
        # lets pick up the line with valuable
        previous_echoed = previous_echoed.splitlines()[-3].strip()
        if previous_echoed.endswith("reserved."):
            return "echo stage-1\r\n"
        if previous_echoed.endswith("stage-1"):
            return "echo stage-2\r\n"
        elif previous_echoed.endswith("stage-2"):
            return "echo stage-3\r\n"
        elif previous_echoed.endswith("stage-3"):
            return "exit\r\n"
        else:
            raise Exception("Unexpected output {0}".format(previous_echoed))
    except IndexError:
        return "echo stage-1\r\n"


class RunFuncTestCase(PexpectTestCase.PexpectTestCase):
    runfunc = staticmethod(wexpect.run)
    cr = '\r'
    empty = ''
    prep_subprocess_out = staticmethod(lambda x: x)

    def test_run_exit(self):
        (data, exitstatus) = self.runfunc('python exit1.py', withexitstatus=1)
        assert exitstatus == 1, "Exit status of 'python exit1.py' should be 1."

    def test_run(self):
        the_old_way = subprocess.Popen(
            args=['uname', '-m', '-n'],
            stdout=subprocess.PIPE
        ).communicate()[0].rstrip()

        (the_new_way, exitstatus) = self.runfunc(
            'uname -m -n', withexitstatus=1)
        the_new_way = the_new_way.replace(self.cr, self.empty).rstrip()

        self.assertEqual(self.prep_subprocess_out(the_old_way).decode('utf-8'), the_new_way)
        self.assertEqual(exitstatus, 0)

    def test_run_callback(self):
        # TODO it seems like this test could block forever if run fails...
        events = {wexpect.TIMEOUT: timeout_callback}
        self.runfunc("cat", timeout=1, events=events)

    def test_run_bad_exitstatus(self):
        (the_new_way, exitstatus) = self.runfunc(
            'ls -l /najoeufhdnzkxjd', withexitstatus=1)
        self.assertNotEqual(exitstatus, 0)

    def test_run_event_as_string(self):
        re_flags =  re.DOTALL | re.MULTILINE
        events = {
            # second match on 'abc', echo 'def'
            re.compile('abc.*>', re_flags): 'echo "def"\r\n',
            # final match on 'def': exit
            re.compile('def.*>', re_flags): 'exit\r\n',
            # first match on 'GO:' prompt, echo 'abc'
            re.compile('Microsoft.*>', re_flags): 'echo "abc"\r\n'
            }

        (data, exitstatus) = wexpect.run(
            'cmd',
            withexitstatus=True,
            events=events,
            timeout=5)
        assert exitstatus == 0

    def test_run_event_as_function(self):
        events = {'>': function_events_callback}

        (data, exitstatus) = wexpect.run(
            'cmd',
            withexitstatus=True,
            events=events,
            timeout=10)
        assert exitstatus == 0

    # Unsupported
    # def test_run_event_as_method(self):
    #     events = {
    #         '>': RunFuncTestCase._method_events_callback
    #     }
    # 
    #     (data, exitstatus) = wexpect.run(
    #         'cmd',
    #         withexitstatus=True,
    #         events=events,
    #         timeout=10)
    #     assert exitstatus == 0

    def test_run_event_typeerror(self):
        events = {'>': -1}
        with self.assertRaises(TypeError):
            wexpect.run('cmd',
                        withexitstatus=True,
                        events=events,
                        timeout=10)

    def _method_events_callback(self, values):
        try:
            previous_echoed = (values["child_result_list"][-1].decode()
                               .split("\n")[-2].strip())
            if previous_echoed.endswith("foo1"):
                return "echo foo2\n"
            elif previous_echoed.endswith("foo2"):
                return "echo foo3\n"
            elif previous_echoed.endswith("foo3"):
                return "exit\n"
            else:
                raise Exception("Unexpected output {0!r}"
                                .format(previous_echoed))
        except IndexError:
            return "echo foo1\n"


if __name__ == '__main__':
    unittest.main()



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.

import os
import sys
from pbr.version import VersionInfo

repo_path = os.path.dirname(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
print(repo_path)
sys.path.insert(0, repo_path)

# import wexpect

os.environ['WEXPECT_SPAWN_CLASS'] = 'SpawnPipe'
autodoc_mock_imports = ["pywintypes", "win32process", "win32con", "win32file", "winerror",
    "win32pipe", "ctypes", "win32console", "win32gui", "psutil"]

# from ctypes import windll

# The master toctree document.
master_doc = 'index'

# -- Project information -----------------------------------------------------

project = 'wexpect'
copyright = '2020, Benedek Racz'
author = 'Benedek Racz'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
package_name='wexpect'
info = VersionInfo(package_name)
version = info.version_string()

# The full version, including alpha/beta/rc tags.
release = version



# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.intersphinx'
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'default'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = []



'''
This is is a very basic stdio handler script. This is used by python.py example.
'''

import time

puskas_wiki = ['''Ferenc Puskas was a Hungarian footballer and manager, widely regarded as one of \
the greatest players of all time. He is the son of former footballer Ferenc Puskas Senior. A \
prolific forward, he scored 84 goals in 85 international matches for Hungary, played 4 \
international matches for Spain and scored 514 goals in 529 matches in the Hungarian and Spanish \
leagues. He became an Olympic champion in 1952 and led his nation to the final of the 1954 World \
Cup where he was named the tournament's best player. He won three European Cups (1959, 1960, 1966),\
 10 national championships (5 Hungarian and 5 Spanish Primera Division) and 8 top individual \
 scoring honors. In 1995, he was recognized as the top scorer of the 20th century by the IFFHS.''',
'''Puskas started his career in Hungary playing for Kispest and Budapest Honved. He was the top scorer\
 in the Hungarian League on four occasions, and in 1948, he was the top goal scorer in Europe. \
 During the 1950s, he was both a prominent member and captain of the Hungarian national team, known\
 as the Mighty Magyars. In 1958, two years after the Hungarian Revolution, he emigrated to Spain \
 where he played for Real Madrid. While playing with Real Madrid, Puskas won four Pichichis and \
 scored seven goals in two European Champions Cup finals.''',
'''After retiring as a player, he became a coach. The highlight of his coaching career came in 1971 \
when he guided Panathinaikos to the European Cup final, where they lost 2-0 to AFC Ajax. In 1993, \
he returned to Hungary and took temporary charge of the Hungarian national team. In 1998, he \
became one of the first ever FIFA/SOS Charity ambassadors. In 2002, the Nepstadion in Budapest \
was renamed the Puskas Ferenc Stadion in his honor. He was also declared the best Hungarian \
player of the last 50 years by the Hungarian Football Federation in the UEFA Jubilee Awards in \
November 2003. In October 2009, FIFA announced the introduction of the FIFA Puskas Award, \
awarded to the player who has scored the "most beautiful goal" over the past year. He was also \
listed in Pele's FIFA 100.''',
 '''Ferenc Purczeld was born on 2 April 1927 to a German (Danube Swabian) family in Budapest and \
brought up in Kispest, then a suburb, today part of the city. His mother, Margit Biro \
(1904-1976), was a seamstress. He began his career as a junior with Kispest AC,[10] where his \
father, who had previously played for the club, was a coach. He had grandchildren, who were the \
children of his brothers son[clarification needed]; the two sons of his brother are Zoltan and \
Istvan, the first one have 3 children; Ilonka, Camila and Andres, and the second one have two.''', 
'''He changed his name to Puskas. He initially used the pseudonym "Miklos Kovacs" to help \
circumvent the minimum age rules[12] before officially signing at the age of 12. Among his early \
teammates was his childhood friend and future international teammate Jozsef Bozsik. He made his \
first senior appearance for Kispest in November 1943 in a match against Nagyvaradi AC.[13] It was \
here where he got the nickname "Ocsi" or "Buddy".[14]''', 
'''Kispest was taken over by the Hungarian Ministry of Defence in 1949, becoming the Hungarian Army \
team and changing its name to Budapest Honved. As a result, football players were given military \
ranks. Puskas eventually became a major (Hungarian: Ornagy), which led to the nickname "The \
Galloping Major".[15] As the army club, Honved used conscription to acquire the best Hungarian \
players, leading to the recruitment of Zoltan Czibor and Sandor Kocsis.[16] During his career at \
Budapest Honved, Puskas helped the club win five Hungarian League titles. He also finished as top \
goal scorer in the league in 1947-48, 1949-50, 1950 and 1953, scoring 50, 31, 25 and 27 goals, \
respectively. In 1948, he was the top goal scorer in Europe.[17]''' ] 

def main():
    print('Welcome!')

    while True:
        print('puskas> ', end='')
        num = input()
        
        if num == 'exit':
            break
        if num == 'all':
            print('\r\n'.join(puskas_wiki))
            continue
        try:
            if int(num) in range(len(puskas_wiki)):
                print(puskas_wiki[int(num)])
                continue
        except:
            pass
        print('unknown command')

if __name__ == '__main__':
    main()
    


from __future__ import annotations

import argparse
import ast
from pathlib import Path


class VersionAnalyzer(ast.NodeVisitor):
    def __init__(self, literal_name: str):
        self.literal_name = literal_name
        self.version = None

    def visit_Assign(self, node: ast.Assign):
        if isinstance(node.targets[0], ast.Name) and node.targets[0].id == self.literal_name:
            if self.version is not None:
                raise ValueError(f"Multiply version assignment for {self.literal_name} found")
            if isinstance(node.value, ast.Constant) and isinstance(node.value.value, str):
                self.version = node.value.value


def cli() -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    parser.add_argument("file", type=str, help="The file to read")
    parser.add_argument("--literal-name", type=str, default="VERSION", help="The literal name to search")
    return parser.parse_args()


def main():
    args = cli()
    with Path(args.file).open("r") as f:
        code = f.read()
    tree = ast.parse(code)
    analyzer = VersionAnalyzer(args.literal_name)
    analyzer.visit(tree)
    version = analyzer.version
    if version is None:
        raise ValueError(f"Version not found for {args.literal_name}")
    print(version)


if __name__ == "__main__":
    main()



import wexpect
import time
import sys
import os

here = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, here)

import i11_unicode_printer

print(wexpect.__version__)

encodings = ['cp1250', 'utf-8']


# With quotes (C:\Program Files\Python37\python.exe needs quotes)
python_executable = '"' + sys.executable + '" '
child_script = here + '\\i11_unicode_printer.py'


def main():
    unicodePrinter = python_executable + ' '  + child_script
    prompt = '> '
    
    for ec in encodings:
        # Start the child process
        p = wexpect.spawn(unicodePrinter)

        # Wait for prompt
        p.expect(prompt)
        print('Child prompt arrived, lets set encoding!')
        p.sendline(ec)
        p.expect(prompt)

        # Send commands
        for cc in range(34, 500):
            p.sendline(str(cc))
            p.expect(prompt)
            print(p.before.splitlines()[1])
            if chr(int(cc)).encode("utf-8").decode(ec) != p.before.splitlines()[1]:
                p.interact()
                time.sleep(5)
                raise Exception()
                 
        
main()
        



import time

# Wait the given time
for i in range(2):
    print('apple\tbanana\tmelone\tcherry')
    print('pepper tomato\tcorn cabbage')
    print('apple banana\tmelone\tcherry2')
    print('pepper tomato\tcorn cabbage2')
    print('prompt> ')
    time.sleep(0.5)




''' This script shows three method to terminate your application. `terminate_programmatically()`
shows the recommended way, which kills the child by sending the application specific exit command.
After that waiting for the terminaton is recommended.
`terminate_eof()` shows how to terminate child program by sending EOF character. Some program can be
terminated by sending EOF character. Waiting for the terminaton is recommended in this case too.
`terminate_terminate()` shows how to terminate child program by sending kill signal. Some
application requires sending SIGTERM to kill the child process. `terminate()` call `kill()`
function, which sends SIGTERM to child process. Waiting for the terminaton is not required
explicitly in this case. The wait is included in `terminate()` function.
'''

import wexpect

def terminate_programmatically():
    '''Terminate child program by command. This is the recommended method. Send your application's
    exit command to quit the child's process. After that wait for the terminaton.
    '''
    print('terminate_programmatically')

    # Start cmd as child process
    child = wexpect.spawn('cmd.exe')

    # Wait for prompt when cmd becomes ready.
    child.expect('>')

    # Exit from cmd
    child.sendline('exit')

    # Waiting for cmd termination.
    child.wait()

def terminate_eof():
    '''Terminate child program by sending EOF character. Some program can be terminated by sending
    an EOF character. Waiting for the terminaton is recommended in this case too.
    '''
    print('terminate_eof')

    # Start cmd as child process
    child = wexpect.spawn('cat')

    # Exit from cmd
    child.sendeof()

    # Waiting for cmd termination.
    child.wait()


def terminate_terminate():
    '''Terminate child program by sending kill signal. Some application requires sending SIGTERM to kill
    the child process. `terminate()` call `kill()` function, which sends SIGTERM to child process.
    Waiting for the terminaton is not required explicitly in this case. The wait is included in
    `terminate()` function.
    '''
    print('terminate_terminate')

    # Start cmd as child process
    child = wexpect.spawn('cmd.exe')

    # Wait for prompt when cmd becomes ready.
    child.expect('>')

    # Exit from cmd
    child.terminate()


terminate_programmatically()
terminate_eof()
terminate_terminate()
